# -*- coding: utf-8 -*-

from datetime import datetime
from functools import wraps


def now():
    return datetime.now().strftime('%Y-%m-%d %H:%M:%S')


def catch_all(func):
    """Simple decorator that wraps a function in a silent tr/except.

    Useful for wrapping functions that might fail but shouldn't propagate that
    failure to the caller. For example, notifications might break, but we don't
    want that to interrupt the iteration.

    """
    @wraps(func)
    def _inner(*args, **kwargs):
        try:
            return func(*args, **kwargs)
        except:
            pass
    return _inner



# -*- coding: utf-8 -*-

import os

from noterator import Noterator


def get_config_path(fname):
    return os.path.join(
        os.path.dirname(os.path.abspath(__file__)),
        'data',
        fname,
    )


def all_available_methods():
    # Take a list of, e.g [EMAIL, HIPCHAT, TWILIO] and turn it into
    # EMAIL|HIPCHAT|TWILIO
    available_methods = [method for (method, _, _) in Noterator.methods]
    method = available_methods.pop(0)
    while len(available_methods):
        method |= available_methods.pop(0)

    return method



# -*- coding: utf-8 -*-

import os

try:
    from ConfigParser import SafeConfigParser as ConfigParser
except ImportError:  # py3k
    import sys
    minor = sys.version_info[1]
    if minor >= 2:
        # SafeConfigParser is deprecated as of 3.2
        from configparser import ConfigParser
    else:
        from configparser import SafeConfigParser as ConfigParser


class ConfigurationError(Exception):
    pass


def load_config(fname=None):
    """Load and return configuration from a file.

    Args:
        fname (str): Path to the ini file we should use. If not provided, we
            default to $HOME/.config/bocho/config.ini

    Returns:
        The parsed configuration

    Raises:
        ConfigurationError: if the file can't be found.

    """
    if not fname:
        fname = os.path.join(
            os.getenv('HOME', ''), '.config', 'noterator', 'config.ini'
        )

    if not os.path.exists(fname):
        raise IOError(
            "Unable to find configuration file."
        )

    config = ConfigParser()
    config.read(fname)

    return config



# -*- coding: utf-8 -*-

import unittest

from .utils import all_available_methods, get_config_path

from noterator import Noterator
from noterator.config import ConfigurationError


class TestConfigValidation(unittest.TestCase):
    def test_valid_config(self):
        noterator = Noterator(
            method=all_available_methods(),
            config_file=get_config_path('config-full.ini')
        )
        noterator._validate_config()

    def test_invalid_config(self):
        noterator = Noterator(
            method=all_available_methods(),
            config_file=get_config_path('config-bad.ini')
        )
        with self.assertRaises(ConfigurationError):
            noterator._validate_config()

    def test_bad_path(self):
        noterator = Noterator(
            config_file='nowhere-useful',
        )
        self.assertEqual(len(noterator.cfg.sections()), 0)



# -*- coding: utf-8 -*-

import mock
import unittest

from noterator import Noterator, HIPCHAT


class TestEmailPlugin(unittest.TestCase):
    @mock.patch('noterator.plugins.hipchat.requests')
    def test_hipchat_settings(self, requests):
        cfg = {
            'token': 'hipchat-token',
            'room_id': '123456',
            'from_name': 'Testy McTesterson',
            'message_colour': 'neon pink',
        }

        n = Noterator(range(5), HIPCHAT, config_file=None)
        n.configure_plugin('hipchat', **cfg)
        for _ in n:
            pass

        url = 'https://api.hipchat.com/v2/room/{}/notification'.format(
            cfg['room_id'],
        )
        payload = {
            "message": "{}: {}".format(n.head, n._get_body(finished=True)),
            "from": cfg['from_name'],
            "color": cfg['message_colour'],
        }
        headers = {
            'Authorization': 'Bearer {}'.format(cfg['token']),
        }

        requests.post.assert_called_once_with(url, payload, headers=headers)



# -*- coding: utf-8 -*-

from __future__ import unicode_literals
from __future__ import absolute_import

from noterator.config import load_config, ConfigParser, ConfigurationError
from noterator.plugins import email, hipchat, twilio
from noterator.utils import catch_all, now

__author__ = 'James Rutherford'
__email__ = 'jim@jimr.org'
__version__ = '0.3.0'


QUIET = 0
EMAIL = 1
TWILIO = 1 << 1
HIPCHAT = 1 << 2


class Noterator(object):
    """Base class for setting up & configuring your noteration.

    You only really need to interact with this class directly (rather than just
    using ``noterator.noterate``) is if you want to build a re-usable
    ``Noterator`` or you want to build one without using a configuration file.

    For example, you could define a ``Noterator`` for use on two iterables, and
    use a different description for each one::

        >>> from noterator import Noterator, EMAIL
        >>> noterator = Noterator(method=EMAIL, every_n=100, start=True)
        >>> for obj in noterator(my_objects, desc="loop 1")
        ...     do_something_slow(obj)
        ...
        >>> for obj in noterator(my_other_objects, desc="loop 2")
        ...     do_something_else_slow(obj)
        ...
        >>> 

    And if you want to build a Noterator without using a configuration file (or
    if you just want to override some configured options)::

        >>> from noterator import Noterator, EMAIL
        >>> noterator = Noterator(my_objects, method=EMAIL, every_n=100)
        >>> noterator.configure_plugin('email', recipient='you@example.org')
        >>> for obj in noterator
        ...     do_something_slow(obj)
        ... 
        >>> 

    If `iterable` isn't set on class initialisation, it must be set before
    looping, or as a parameter to the ``noterate`` method.

    When iteration begins, the configuration will be validated.

    Args:
        Technically none, but ``iterable``, ``method``, and ``desc`` are often
        passed as positional arguments, and such usage is encouraged for
        brevity.

    Kwargs:
        iterable (iterable): The iterable we're going to wrap
        desc (str): Description to include in the notification.
        start (bool): Send a notification when iteration starts
        finish (bool): Send a notification when iteration completes
        every_n (int): Send a notification every ``every_n`` iterations
        method (int): Method(s) to use (e.g. ``EMAIL|HIPCHAT``)
        head (string): Header for notification message
        body (string): Body for notification message. We call `.format` on this
                       string with the iteration count at the point of
                       notification.
        config_file (string): Path to alternative configuration file.

    """
    index = 0
    methods = (
        (EMAIL, 'email', email),
        (TWILIO, 'twilio', twilio),
        (HIPCHAT, 'hipchat', hipchat),
    )

    def __init__(self, iterable=None, method=QUIET, desc=None, head=None,
                 body=None, start=False, finish=True, every_n=None,
                 config_file=None):
        self.iterable = iterable
        self.method = method

        self.desc = desc or 'Iteration'
        self.head = head or 'Noterator alert'
        self.body = body or 'progress: {} iterations.'

        self.start = start
        self.finish = finish
        self.every_n = every_n

        # If the configuration file isn't present, build an empty configuration
        # that can be populated with `self.configure_plugin`.
        self.cfg = ConfigParser()
        try:
            self.cfg = load_config(config_file)
        except IOError:
            pass

    def __call__(self, iterable=None, method=None, desc=None, **kwargs):
        """Allow any parameter to be set at execution time.

        For example::

            >>> from noterator import Noterator, EMAIL
            >>> noterator = Noterator(my_objects, EMAIL, every_n=100)
            >>> for obj in noterator(desc="loop 1")
            ...     do_something_slow(obj)
            ... 

        """
        if iterable is not None:
            self.iterable = iterable

        if method is not None:
            self.method = method

        if desc is not None:
            self.desc = desc

        for key, value in kwargs.items():
            setattr(self, key, value)

        return self

    def __iter__(self):
        self._validate_config()
        self._notify(started=True)

        for obj in self.iterable:
            yield obj
            self._notify()
            self.index += 1

        self._notify(finished=True)

    def next(self):
        if self.index == 0:
            self._validate_config()
            self._notify(started=True)

        try:
            result = next(self.iterable)
            self.index += 1
            self._notify()
            return result
        except StopIteration:
            self._notify(finished=True)
            raise

    def configure_plugin(self, name, **kwargs):
        """Optionally set up plugins without using a configuration file.

        You can also use this to override defaults set in a configuration file.

        Args:
            name (str): the name of the plugin (e.g. 'email')

        Kwargs: any keyword arguments accepted by the named plugin.

        """
        if not self.cfg.has_section(name):
            self.cfg.add_section(name)

        for key, value in kwargs.items():
            self.cfg.set(name, key, value)

    def _validate_config(self):
        for flag, config_key, module in self.methods:
            if self.method & flag:
                # If we're using a given notification method, make sure it's
                # configured.
                if config_key not in self.cfg.sections():
                    raise ConfigurationError(
                        '{} is not configured'.format(config_key),
                    )
                else:
                    # Make sure all required configuration parameters are set
                    configured = set(self.cfg.options(config_key))
                    required = set(module.REQUIRED_CONFIG)
                    if configured & required != required:
                        raise ConfigurationError(
                            '{} is missing required settings: {}'.format(
                                config_key, ', '.join(required - configured),
                            )
                        )

    @catch_all
    def _notify(self, started=False, finished=False):
        send = False
        body = self._get_body(started, finished)

        if self.start and started:
            send = True

        if self.finish and finished:
            send = True
        elif self.every_n:
            # We don't send `every_n` notifications on iteration completion
            # where that would be a duplicate notification hence the `elif`
            # with `self.finish and finished`
            if self.index > 0 and self.index % self.every_n == 0:
                send = True

        if send:
            for method, config_key, module in self.methods:
                if self.method & method:
                    module.notify(
                        self.head, body, **dict(self.cfg.items(config_key))
                    )

    def _get_body(self, started=False, finished=False):
        body = self.body.format(self.index)
        if started:
            body = '{} started at {}.'.format(self.desc, now())

        if finished:
            body = '{} finished at {} (total iterations: {}).'.format(
                self.desc, now(), self.index,
            )
        elif self.every_n and self.index > 0:
            body = '{} completed {} iterations at {}.'.format(
                self.desc, self.index, now(),
            )

        return body


def noterate(iterable, *args, **kwargs):
    """Wrap any iterable with `noterate` and you'll be notified when the iteration
    completes, for example::

        >>> from noterator import noterate, EMAIL
        >>> for obj in noterate(my_objects, EMAIL, "My super-slow iteration"):
        ...     do_something_slow(obj)
        ... 
        >>> 

    By default you only find out when the iteration completes, but sometimes
    it's useful to know when these things start too::

        >>> from noterator import noterate, EMAIL, HIPCHAT
        >>> for obj in noterate(my_objects, EMAIL|HIPCHAT, start=True):
        ...     do_something_slow(obj)
        ... 
        >>> 

    This function is a convenience wrapper around the ``Noterator`` class. The
    code above is equivalent to::

        >>> from noterator import Noterator, EMAIL, HIPCHAT
        >>> noterator = Noterator(my_objects, EMAIL|HIPCHAT, start=True)
        >>> for obj in noterator:
        ...     do_something_slow(obj)
        ... 
        >>> 

    Positional and Keyword arguments are the same as for ``Noterator.__init__``
    except that ``iterable`` is required.

    """
    return Noterator(iterable, *args, **kwargs)



# -*- coding: utf-8 -*-

import mock
import unittest

from email.mime.text import MIMEText

from noterator import Noterator, EMAIL


class TestEmailPlugin(unittest.TestCase):
    @mock.patch('noterator.plugins.email.smtplib')
    def test_email_settings(self, smtplib):
        cfg = {
            'recipient': 'to@example.com',
            'from_mail': 'from@example.com',
            'host': 'smtp.example.com',
            'port': '250',  # not 25 because that's the default value
            'username': 'smtpuser',
            'password': 'smtppass',
        }

        smtp = mock.Mock()
        smtplib.SMTP = mock.Mock(return_value=smtp)

        n = Noterator(range(5), EMAIL, config_file=None)
        n.configure_plugin('email', **cfg)
        for _ in n:
            pass

        msg = MIMEText(n._get_body(finished=True))
        msg['Subject'] = n.head
        msg['From'] = cfg['from_mail']
        msg['To'] = cfg['recipient']

        smtplib.SMTP.assert_called_once_with(cfg['host'], cfg['port'])
        smtp.sendmail.assert_called_once_with(
            cfg['from_mail'], [cfg['recipient']], msg.as_string(),
        )



# -*- coding: utf-8 -*-

from __future__ import absolute_import

import requests

REQUIRED_CONFIG = [
    'account_sid', 'token', 'from_number', 'to_number',
]


def notify(head, body, **kwargs):
    account_sid = kwargs['account_sid']
    token = kwargs['token']
    from_number = kwargs['from_number']
    to_number = kwargs['to_number']

    url = 'https://api.twilio.com/2010-04-01/Accounts/{}/Messages.json'.format(
        account_sid,
    )
    payload = {
        "From": from_number,
        "To": to_number,
        "Body": "{}: {}".format(head, body),
    }
    auth = (account_sid, token)

    requests.post(url, payload, auth=auth)



# -*- coding: utf-8 -*-

from __future__ import absolute_import

import smtplib
from email.mime.text import MIMEText

REQUIRED_CONFIG = [
    'from_mail', 'recipient', 'host',
]


def notify(head, body, **kwargs):
    from_mail = kwargs['from_mail']
    recipient = kwargs['recipient']

    msg = MIMEText(body)
    msg['Subject'] = head
    msg['From'] = from_mail
    msg['To'] = recipient

    s = smtplib.SMTP(kwargs['host'], kwargs.get('port', 25))
    if kwargs['username'] and kwargs['password']:
        s.login(kwargs['username'], kwargs['password'])

    s.sendmail(from_mail, [recipient], msg.as_string())
    s.quit()



# -*- coding: utf-8 -*-

from __future__ import absolute_import

import requests

REQUIRED_CONFIG = [
    'token', 'room_id',
]


def notify(head, body, **kwargs):
    token = kwargs['token']
    room_id = kwargs['room_id']
    from_name = kwargs.get('from_name', 'The Noterator')
    message_colour = kwargs.get('message_colour', 'green')

    url = 'https://api.hipchat.com/v2/room/{}/notification'.format(room_id)
    payload = {
        "message": "{}: {}".format(head, body),
        "from": from_name,
        "color": message_colour,
    }
    headers = {
        'Authorization': 'Bearer {}'.format(token),
    }

    requests.post(url, payload, headers=headers)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from setuptools import setup

with open('README.rst') as readme_file:
    readme = readme_file.read()

with open('HISTORY.rst') as history_file:
    history = history_file.read()

requirements = [
    'requests[security]',
]

test_requirements = [
    'mock==2.0.0',
]

setup(
    name='noterator',
    version='0.3.0',
    description="The Noterator: bringing notification to iteration.",
    long_description=readme + '\n\n' + history,
    author="James Rutherford",
    author_email='jim@jimr.org',
    url='https://github.com/jimr/noterator',
    packages=['noterator'],
    package_dir={'noterator': 'noterator'},
    include_package_data=True,
    install_requires=requirements,
    license="MIT license",
    zip_safe=False,
    keywords='noterator',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        "Programming Language :: Python :: 2",
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
    ],
    test_suite='tests',
    tests_require=test_requirements
)



# -*- coding: utf-8 -*-

import mock
import unittest

from noterator import Noterator, TWILIO


class TestEmailPlugin(unittest.TestCase):
    @mock.patch('noterator.plugins.twilio.requests')
    def test_twilio_settings(self, requests):
        cfg = {
            'account_sid': '123456',
            'token': 'twilio-token',
            'from_number': '+987654',
            'to_number': '+13579',
        }

        n = Noterator(range(5), TWILIO, config_file=None)
        n.configure_plugin('twilio', **cfg)
        for _ in n:
            pass

        url = 'https://api.twilio.com/2010-04-01/Accounts/{}/Messages.json'.format(
            cfg['account_sid'],
        )
        payload = {
            "From": cfg['from_number'],
            "To": cfg['to_number'],
            "Body": "{}: {}".format(n.head, n._get_body(finished=True)),
        }
        auth = (cfg['account_sid'], cfg['token'])

        requests.post.assert_called_once_with(url, payload, auth=auth)
